package id.co.ecoin.test.objectdb;

//import com.google.common.base.Preconditions;
import javax.jdo.annotations.Unique;
import javax.persistence.*;
import java.util.*;

import static org.junit.Assert.assertTrue;

public class ObjectDbTest {
    EntityManager em;

    Random rnd = new Random();

    public static void main(String [] args){
        ObjectDbTest t = new ObjectDbTest();

        t.init();
        t.objectDbBasic2();
    }

    public static EntityManager getTestManager(){
        Map<String, String> properties = new HashMap<String, String>();
        properties.put("javax.persistence.jdbc.user", "admin");
        properties.put("javax.persistence.jdbc.password", "admin");
        EntityManagerFactory emf = Persistence.createEntityManagerFactory(
                "objectdb://10.11.12.3:6136/test.odb", properties);

        return emf.createEntityManager(properties);
    }

    public static EntityManager getLocalManager(){
        String dbfile = "$objectdb/db/test.odb";
        EntityManagerFactory emf =
                Persistence.createEntityManagerFactory(dbfile);
        return emf.createEntityManager();
    }
    public void init(){
        em = getLocalManager();
        em.getTransaction().begin();
        int deletedCount = em.createQuery("DELETE FROM Object").executeUpdate();
        em.getTransaction().commit();
    }

    public void objectDbBasic2(){
        assertTrue(em != null);

        byte[] r = new byte[16];

        LinkedList<byte[]> keys = new LinkedList<>();

        for (int i = 0 ; i < 100; i++){
            rnd.nextBytes(r);
            keys.add(r);
            OdbByteArray bai = OdbByteArray.newInstance(em, r);

            OdbByteArray oba = OdbByteArray.find(em, r);

            assertTrue(Arrays.equals(bai.key, oba.key));
        }

        for (byte[] key : keys){
            OdbByteArray bai = OdbByteArray.find(em, key);
            assertTrue(Arrays.equals(bai.key, key));
        }

        for (int i = 0; i< 100; i++){
            rnd.nextBytes(r);
            OdbByteArray bai = OdbByteArray.find(em, r);
            assertTrue(bai == null);
        }
    }

    @Entity(name="OdbByteArray")
    public static class OdbByteArray {
        //@Id
        //@GeneratedValue
        //long id;

        //@Unique
        @Unique
        public byte[] key;
        public int test;

        public static OdbByteArray newInstance(EntityManager em, byte[] key){
            System.out.println(Base64.getEncoder().encodeToString(key));
            if (find(em, key) != null) return null;

            OdbByteArray bai = new OdbByteArray();
            bai.key = key;

            em.getTransaction().begin();
            em.persist(bai);
            em.getTransaction().commit();

            return bai;
        }

        public static OdbByteArray find(EntityManager em, byte[] key){
            //Preconditions.checkNotNull(key);
            TypedQuery<OdbByteArray> query = em.createQuery(
                    "SELECT e FROM OdbByteArray e WHERE e.key = :key", OdbByteArray.class);
            query.setParameter("key", key);
            List<OdbByteArray> res = query.getResultList();
            if (res.isEmpty()) return null;
            return res.get(0);
        }
    }
}
